# -*- coding: utf-8 -*-

from .log import logger


class Request(object):
    """
    请求
    """

    worker = None
    box_class = None
    msg = None
    box = None
    is_valid = False

    def __init__(self, worker, box_class, msg):
        self.worker = worker
        self.box_class = box_class
        self.msg = msg
        self.is_valid = self._parse_raw_data()

    def _parse_raw_data(self):
        try:
            self.box = self.box_class()
        except Exception, e:
            logger.error('parse raw_data fail. e: %s, request: %s', e, self)
            return False

        if self.box.unpack(self.msg.get('data')) > 0:
            return True
        else:
            logger.error('unpack fail. request: %s', self)
            return False

    @property
    def app(self):
        return self.worker.app

    @property
    def blueprint(self):
        cmd_parts = str(self.cmd or '').split('.')
        bp_name, cmd = cmd_parts if len(cmd_parts) == 2 else (None, self.cmd)

        for bp in self.app.blueprints:
            if bp_name == bp.name and bp.get_route_view_func(cmd):
                return bp

        return None

    @property
    def blueprint_cmd(self):
        cmd_parts = str(self.cmd or '').split('.')
        return cmd_parts[1] if len(cmd_parts) == 2 else self.cmd

    @property
    def address(self):
        return self.msg.get('address')

    @property
    def cmd(self):
        try:
            return self.box.cmd
        except:
            return None

    def write(self, data):
        if isinstance(data, dict):
            # 生成box
            data = self.box.map(data)

        if isinstance(data, self.box_class):
            data = data.pack()

        return self.worker.output_queue.put(dict(
            conn_id=self.msg.get('conn_id'),
            data=data,
        ))

    def close(self, exc_info=False):
        """
        还没做，也没想好怎么做
        :param exc_info:
        :return:
        """
        self.write(None)

    def __repr__(self):
        return repr(self.msg)
